<?php

/**
 * @file
 * Contains \Drupal\ces_import\ImportAll.
 *
 */
namespace Drupal\ces_import;

use Drupal\Core\Form\FormStateInterface;

class ImportAll extends \Drupal\Core\Form\FormBase {

  protected $bundle = 'want';

  protected $moduleInstaller;
  protected $configFactory;

  function __construct($module_installer, $config_factory) {
    $this->moduleInstaller = $module_installer;
    $this->configFactory = $config_factory;
  }

  static function create(\Symfony\Component\DependencyInjection\ContainerInterface $container) {
    return new static(
      $container->get('module_installer'),
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'ces_import_all_form';
  }


  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['title'] = [
      '#markup' => $this->t(
        'Import from Australian CES exchange: @name',
        ['@name' => $this->configFactory->get('ces.settings')->get('code')]
      )
    ];

    $form['pass'] = [
      '#title' => 'Password',
      '#description' => 'The password of account 0000',
      '#type' => 'textfield',
      '#required' => TRUE,
      '#max_length' => 20,
      '#size' => 10,
      '#weight' => 1,
      '#required' => TRUE
    ];

    $form['uninstall'] = array(
      '#title' => t('Uninstall'),
      '#type' => 'checkbox',
      '#default_value' => TRUE,
      '#weight' => 2
    );

    $form['submit'] = array (
      '#type' => 'submit',
      '#value' => 'import',
      '#weight' => 3
    );
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    ces_import_australia(
      \Drupal::config('ces.settings')->get('code'),
      $form_state->getValue('pass')
    );

    if ($form_state->get('uninstall')) {
      $this->uninstall(['ces_import']);
    }
  }

}