<?php

/**
 * @file
 * Contains \Drupal\ces_import\ImportTransactions.
 *
 */
namespace Drupal\ces_import;
use Drupal\mcapi\Entity\TransactionInterface;

class ImportTransactions extends ImportBase {

  protected $entityTypeId = 'mcapi_transaction';
  protected $bundle = 'mcapi_transaction';

  private $allWallets = [];
  protected $temp = NULL;

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'ces_import_transactions_form';
  }

  function columns() {
    return [
      'payer',// wallet id e.g. abcd1234
      'payee',// wallet id e.g. abcd1234
      'quantity', //integer number of talents
      'created', //item created date, format:  "2016-01-15 19:10:33"
      'state', //is always 1 which translates to TRANSACTION_STATE_DONE in D8
      'type', //last changed date, format:  "2016-01-15 19:10:33"
      'description', //text
      //'categories',
    ];
  }

  public function deleteAll() {
    foreach ($this->entityTypeManager->getStorage('mcapi_transaction')->loadMultiple() as $entity) {
      $entity->delete();
    }
  }

  protected function stateProcess(TransactionInterface $entity, $val, $allfields) {
    $entity->state->target_id  = TRANSACTION_STATE_FINISHED;
  }

  protected function typeProcess(TransactionInterface $entity, $val, $allfields) {
    $entity->type->target_id  = strtolower($val);
  }

  protected function quantityProcess(TransactionInterface $entity, $val, $allfields) {
    $currency = \Drupal\mcapi\Entity\Currency::load('cc');
    @list($talents, $cents) = explode('.', $val);
    $entity->worth->curr_id = 'cc';
    $entity->worth->value = $currency->unformat([1 => $talents, 3 => $cents]);
  }

  protected function payerProcess(TransactionInterface $entity, $val, $allfields) {
    $entity->payer->target_id = $this->getWalletId($val);
  }
  protected function payeeProcess(TransactionInterface $entity, $val, $allfields) {
    $entity->payee->target_id = $this->getWalletId($val);
  }

  protected function getWalletId($name) {
    if (!isset($this->allWallets[$name])) {
      //assumes no two wallet names are the same
      $wids = $this->entityTypeManager
        ->getStorage('mcapi_wallet')->getQuery()
        ->condition('name', '%'.$name.'%', 'LIKE')
        ->execute();
      if ($wids) {
        $this->allWallets[$name] = reset($wids);
      }
      else {
        //create a new wallet, owned by user 1
        $wallet = \Drupal\mcapi\Entity\Wallet::create(['name' => $name, 'orphaned' => TRUE, 'created' => 0]);
        $wallet->save();
        $this->allWallets[$name] = $wallet->id();
      }
    }
    return $this->allWallets[$name];
  }

 /**
  * batch callback
  * @param type $rows
  * @param type $context
  */
  function saveEntities($rows, &$context) {
    foreach ($rows as $rownum => $row) {
      $this->stash($this->buildEntity($row));
    }
  }

  //we have to use the tempstore here because both static variables in the method
  //and object properties kept being reset to this object itself
  function stash($transaction) {

    if ($prev = $this->tempStore->get('saved')) {
      if ($transaction->type->target_id == 'default') {
        $prev->save();
        $this->tempStore->set('saved', $transaction);
      }
      elseif ($transaction->type->target_id == 'auto') {
        $prev->children[] = $transaction;
        $this->tempStore->set('saved', $prev);
      }
    }
    elseif($transaction->type->target_id == 'default') {
      $this->tempStore->set('saved', $transaction);
    }
    else {
      dsm('problem');
    }
  }

  function saveLastTransaction($success, $results, $operations) {
    //trigger saving the last transaction
    //ugly workaround because the sandbox appears not to be working
     $this->tempStore->get('saved')->save();
  }

  function makeBatch($rows, $delete = FALSE) {
    $batch = parent::makeBatch($rows, $delete);
    $batch['title'] = 'Importing transaction history';
    $batch['finished'] = [$this, 'saveLastTransaction'];
    return $batch;
  }



}
