<?php

/**
 * @file
 * Contains \Drupal\cforge\Installer\Setup2.
 */

namespace Drupal\cforge\Installer;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\field\Entity\FieldConfig;
use Drupal\address\Entity\AddressFormat;


/**
 * Choose between different Cforge flavours
 */
class Setup1 extends FormBase {

 private $installer;

  public function __construct($module_installer) {
    $this->installer = $module_installer;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('module_installer')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId()  {
    return 'cforge_setup_1_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['#title'] = $this->t('Setup stage 1');
    $form['flavour'] = [
      '#type' => 'radios',
      '#options' => [
        'hamlets' => $this->t('Hamlets'),
        'ces' => $this->t('CES standalone site'),
      ],
      '#required' => TRUE
    ];

    $country = strtoupper(\Drupal::config('system.date')->get('country.default'));
    $options = \Drupal::service('address.country_repository')->getList();

    if (isset($options[$country])) {
      $form['country'] = [
        '#type' => 'value',
        '#value' => $country
      ];
    }
    else {
      $form['country'] = [
        '#type' => 'select',
        '#title' => t('Country'),
        '#options' => $options,
        '#required' => TRUE,
        '#size' => 12,
      ];
    }

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => 'submit'
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    //finally set the addressfield to the country's addressFormat
    $country_code = $form_state->getValue('country');
    $field = FieldConfig::load('user.user.address');
    $settings = $field->get('settings');
    $settings['available_countries'][$country_code] = $country_code;
    $field->set('settings', $settings)->save();
    modify_address_format($country_code);
    $this->installer->install([$form_state->getValue('flavour')]);

    //@todo ensure the defaultcontent module is in the composer file
    $menuLinkStorage = \Drupal::entityTypeManager()->getStorage('menu_link_content');
    $this->installer->install(['defaultcontent']);
    $this->installer->uninstall(['defaultcontent', 'hal', 'serialization']);
    //to get around a bug in defaultContent which fails to export Path aliases
    //we're going to set drupal's homepage to the one with the menu link called Welcome

    $welcomeLinks = $menuLinkStorage->loadByProperties(['title' => 'Welcome']);
    $url = current($welcomeLinks)->getUrlObject();
    $config = \Drupal::configFactory()->getEditable('system.site');
    $temp = $config->get('page');
    //the crux of the matter
    $temp['front'] = '/'.$url->getInternalPath();//because the imported node doesn't have an alias
    $config->set('page', $temp)->save();
    //@todo see if the home page was correctly set
    drupal_set_message('Set home page to '.$temp['front']);

  }

}


function modify_address_format($country_code) {
  $format = AddressFormat::load($country_code);
  if (!$format) {
    echo 'failed to load country '.$country_code;
  }
  //if we have an altered version of the address, overwrite that country's address with ours
  if ($alt = AddressFormat::load('CF-'.$country_code)) {
    foreach ($alt->toArray() as $key => $value) {
      if ($key != 'countryCode' && $key != 'uuid') {
        $format->set($key, $value);
      }
    }
    $format->save();
  }
  //if we don't then bodge the existing format
  else {
    $pattern = $format->getFormat();
    $pattern .="\n%dependentLocality";
    $format->setFormat($pattern)
      ->setrequiredFields(['dependentLocality'])
      ->save();
  }
}
